(****************************************************************************)   
(** <MKSoftware> - Software fr den Unterricht von Michael Khn *)   
(** www.mksoftware.de.vu *)
(** mail: mksoftware@bm-kuehn.de *)
(****************************************************************************)   
(* Textdatei.mod (Mrz 2002) *)
(* (c) by Michael Khn *)
(**                                                                          *)
(** Dieses Service-Modul darf fr Unterrichtszwecke frei verwendet werden.   *)
(**                                                                          *)
(** Fr Verbesserungsvorschlge bin ich dankbar.                             *)
(****************************************************************************)   

(** *)
(** <Textdatei>*)
(** Lesen und Schreiben von Textdateien*)
(** *)
(** Dieses Modul ermglicht auf etwas vereinfachte Weise gegenber dem File-Modul
    den Umgang mit Textdateien. Es kann eine Textdatei gelesen und geschrieben 
    werden. Dies geschieht zeilenweise. Fr jede Zeile muss ein Lese- bzw. ein 
    Schreib-Befehl ausgefhrt werden. Mehrere Dateien knnen mit diesem Modul nicht
    gleichzeitig geffnet sein, das ist der Preis fr die Einfachheit...
*)
(** *)
(** <Wichtig>: Die Datei muss mit einem Zeilenumbruch abgeschlossen sein, sonst wird
    die letzte Zeile nicht erkannt und es gibt eine Fehlermeldung!*)
MODULE Textdatei; 
IMPORT D:=Display, F:=File;

CONST nurLesen*          = 0;
      (** Diese Konstanten sind fr den <Modus> in der Prozedur <Oeffne> wichtig. *)
      nurSchreiben*      = 1;
      lesenUndSchreiben* = 2;

TYPE TDateiname* = ARRAY 255 OF CHAR;
     TModus* = SHORTINT;
     
VAR  dathandle: F.Handle;
     
PROCEDURE Oeffne*(datName: TDateiname; modus: TModus; VAR ok: BOOLEAN);
(** ffnet die Textdatei, deren Dateiname in <datName> angegeben wurde. *)
(** <modus> gibt an, ob die Datei nur zum Lesen, nur zum Schreiben oder zu beidem
    geffnet werden soll. Dazu verwendet man die Konstanten <nurLesen>, <nurSchreiben>
    bzw. <lesenUndSchreiben>.*)
(** In der Variablen <ok> wird zurckgegeben, ob die Datei erfolgreich 
    geffnet werden konnte.*)
VAR resCode: INTEGER;
    message: ARRAY 255 OF CHAR;
    zeichen: CHAR;
BEGIN
  ok := TRUE;
  IF ~F.Exist(datName) 
    THEN
      D.WriteStr("Fehler! Die Datei ");
      D.WriteStr(datName);
      D.WriteStr(" existiert nicht!");
      D.WriteLn;
      ok:= FALSE;
    ELSE
      CASE modus OF
      | nurLesen:          F.Open(datName,FALSE,F.DENYWRITE,F.READONLY,dathandle,resCode);
      | nurSchreiben:      F.Open(datName,FALSE,F.DENYALL,F.WRITEONLY,dathandle,resCode);
      | lesenUndSchreiben: F.Open(datName,FALSE,F.DENYALL,F.READWRITE,dathandle,resCode);
      ELSE resCode := F.ACCESSDENIED;
      END; 
      IF resCode # F.NOERROR 
      THEN F.GetErrorMessage(resCode,message);
           D.WriteStr("Fehler beim ffnen der Datei ");
           D.WriteStr(datName);
           D.WriteStr(": ");
           D.WriteStr(message);
           D.WriteLn;
           ok:= FALSE;
      END;
  END;
END Oeffne;

PROCEDURE Schliesse*;
(** Die geffnete Textdatei wird wieder geschlossen. Da nur eine Datei zurzeit geffnet
    sein kann, bentigt man keine weiteren Angaben.*)
BEGIN
  F.Close(dathandle);
END Schliesse;

PROCEDURE LiesZeile*(VAR zeile: ARRAY OF CHAR; VAR ok: BOOLEAN);
(** Die nchste Zeile der Datei wird eingelesen und im String gespeichert. 
    In <ok> wird gespeichert, ob das Lesen geklappt hat. *)
VAR resCode: INTEGER;
    message: ARRAY 255 OF CHAR;
BEGIN
  ok := TRUE;
  F.ReadLn(dathandle, zeile, resCode);
  
  IF resCode # F.NOERROR 
  THEN 
    F.GetErrorMessage(resCode,message);
    D.WriteStr("Fehler beim Lesen einer Zeile: ");
    D.WriteStr(message);
    D.WriteLn;
    ok:= FALSE;
  END;
END LiesZeile;

PROCEDURE SchreibeZeile*(zeile: ARRAY OF CHAR; VAR ok: BOOLEAN);
(** Die nchste Zeile der Datei wird geschrieben. 
    In <ok> wird gespeichert, ob das Schreiben geklappt hat. *)
VAR resCode: INTEGER;
    message: ARRAY 255 OF CHAR;
BEGIN
  ok := TRUE;
  F.WriteLn(dathandle, zeile, resCode);
  
  IF resCode # F.NOERROR 
  THEN 
    F.GetErrorMessage(resCode,message);
    D.WriteStr("Fehler beim Schreiben einer Zeile: ");
    D.WriteStr(message);
    D.WriteLn;
    ok:= FALSE;
  END;
END SchreibeZeile;

PROCEDURE Dateiende*(): BOOLEAN;
(** Diese Funktion muss mit den leeren Klammern aufgerufen werden! Es wird berprft,
    ob der Dateizeiger am Ende der Datei steht.*)
BEGIN
  RETURN F.AtEnd(dathandle);
END Dateiende;

END Textdatei.
