(****************************************************************************)   
(** <MKSoftware> - Software fr den Unterricht von Michael Khn *)
(** www.mksoftware.de.vu *)
(** mail: mksoftware@bm-kuehn.de *)
(****************************************************************************)   
(* Textdatei2Matrix.mod (Mrz 2002) *)
(* (c) by Michael Khn *)
(**                                                                          *)
(** Dieses Service-Modul darf fr Unterrichtszwecke frei verwendet werden.   *)
(**                                                                          *)
(** Fr Verbesserungsvorschlge bin ich dankbar.                             *)
(****************************************************************************)   

(** *)
(** <Textdatei2Matrix>*)
(** Lesen und Schreiben von Textdateien*)
(** *)
(** Dieses Modul ermglicht auf vereinfachte Weise gegenber dem File-Modul
    den Umgang mit Textdateien. Es knnen Textdateien gelesen und 
    geschrieben werden. Im Anwendungsprogramm wird der gesamte Text "in einem 
    Rutsch" in einer Matrix aus Zeichen gespeichert.
*)
(** *)
(** Wichtig: Die Datei muss mit einem Zeilenumbruch abgeschlossen sein, sonst wird
    die letzte Zeile nicht erkannt!*)
MODULE Textdatei2Matrix; 
 (* stellt elementare Operationen fr Text-Dateien zur Verfgung.
    Eine Datei wird zeilenweise in eine Matrix aus Buchstaben geschrieben.
    Es wird beim Index 0 begonnen. 
    Ist nach dem Einlesen anzZeilen=-1 bedeutet dies "Fehler" 
    Ansonsten gibt "anzZeilen" wirklich die Anzahl der Zeilen wieder,
    der letzte Index ist also um eins kleiner.                         *)
IMPORT D:=Display, F:=File;

CONST (** Die Konstante <maxZeilen> muss evtl. im Rahmen der
          Speicherkapazitten angepasst werden! *)
      maxZeilen- = 100;
      maxZeichen- = 100; (* pro Zeile *)
      (** Die Konstante <maxZeichen> (pro Zeile) muss evtl. im Rahmen der
          Speicherkapazitten angepasst werden! *)
      max = maxZeilen*maxZeichen;
TYPE TDateiname* = ARRAY 255 OF CHAR;
     TString* = ARRAY maxZeichen OF CHAR;
     TMatrix* = ARRAY maxZeilen OF TString;
     
PROCEDURE LiesTextdatei*(datName: TDateiname; VAR M: TMatrix; 
                         VAR anzZeilen: LONGINT; VAR ok: BOOLEAN);
(** Liest eine Textdatei, deren Name in <datName> angegeben wurde. 
    Nach diesem Prozeduraufruf steht der Text der Datei in der Matrix <M>. 
    Der erste Index ist jeweils 0.
    In jeder Zeile der Matrix (1. Index) steht jeweils eine Zeile der Datei. 
    Auf einen solchen String kann z. B. mit M[2] zugegriffen werden, im 
    Beispiel also auf die 3. Zeile.*)
(** In der Variablen <anzZeilen> wird zurckgegeben, wie viele Zeilen gelesen wurden,
    dabei entspricht dies der tatschlichen Anzahl, die letzte Zeile hat also den
    Index <anzZeilen>-1. Die letzte Zeile der Datei muss mit einem Zeilenumbruch
    abgeschlossen sein, damit auch sie erkannt wird!*)
(** In der Variablen <ok> wird zurckgegeben, ob die Datei erfolgreich 
    gelesen werden konnte. Sollte die Datei mehr Zeilen als maxZeilen haben,
    so werden die ersten maxZeilen dennoch belegt, <ok> wird aber den Wert
    FALSE haben.*)
VAR zaehler, anzZeichen: LONGINT;
    resCode: INTEGER;
    message: ARRAY 255 OF CHAR;
    dat: F.Handle;
    zeichen: CHAR;
BEGIN
  ok := TRUE;
  anzZeilen  := -1;
  anzZeichen := -1;
  IF ~F.Exist(datName) 
    THEN
      D.WriteStr("Fehler! Die Datei ");
      D.WriteStr(datName);
      D.WriteStr(" existiert nicht!");
      D.WriteLn;
      ok:= FALSE;
    ELSE
      F.Open(datName,FALSE,F.DENYWRITE,F.READONLY,dat,resCode);
      IF resCode # F.NOERROR 
      THEN F.GetErrorMessage(resCode,message);
           D.WriteStr("Fehler beim ffnen der Datei ");
           D.WriteStr(datName);
           D.WriteStr(": ");
           D.WriteStr(message);
           D.WriteLn;
           ok:= FALSE;
      ELSE IF ~F.AtEnd(dat)        (* ffnen hat geklappt *)
           THEN INC(anzZeilen);
           END;
           WHILE (~F.AtEnd(dat)) 
                 & (anzZeilen < maxZeilen) 
                 & (anzZeichen < maxZeichen) DO
             (* Die Datei wird zeichenweise eingelesen, um eine berprfung 
                auf Zeilenlnge vornehmen zu knnen *)                 
             F.ReadChar(dat,zeichen,resCode);
             IF resCode # F.NOERROR 
             THEN F.GetErrorMessage(resCode,message);
                  D.WriteStr("Fehler beim ffnen der Datei ");
                  D.WriteStr(datName);
                  D.WriteStr(": ");
                  D.WriteStr(message);
                  D.WriteLn;
                  ok:= FALSE;
             ELSE (* in Matrix einfgen *)
               INC(anzZeichen);
               IF zeichen = CHR(13) 
               THEN (* neue Zeile beginnt *)
                 F.ReadChar(dat,zeichen,resCode); (* jetzt folgt noch Zeichen CHR(10) *)
                 M[anzZeilen][anzZeichen] := 0X;
                 anzZeichen := -1;
                 INC(anzZeilen); (* bisher keine Fehlerabfrage auf >maxZeilen *)
               ELSE (* alte Zeile geht weiter *)
                 M[anzZeilen][anzZeichen] := zeichen;
               END;
             END
           END;
           IF (anzZeilen = maxZeilen) & (~F.AtEnd(dat)) THEN
             D.WriteStr("Datei ");
             D.WriteStr(datName);
             D.WriteStr(" ist zu lang.");
             D.WriteLn;
             D.WriteStr(" Es werden nur die ersten ");
             D.WriteInt(maxZeilen,1);
             D.WriteStr(" Zeilen bercksichtigt!");
             D.WriteLn;
             ok:= FALSE;
           END;
           IF (anzZeichen = maxZeichen) & (~F.AtEnd(dat)) THEN
             D.WriteStr("Zeile ");
             D.WriteInt(anzZeilen,1);
             D.WriteStr(" ist zu lang.");
             D.WriteLn;
             D.WriteStr(" Zeilen drfen max.  ");
             D.WriteInt(maxZeichen,1);
             D.WriteStr(" Zeichen lang sein!");
             D.WriteLn;
             ok:= FALSE;
           END;
      END;
      F.Close(dat);
  END;
END LiesTextdatei;

PROCEDURE SchreibeTextdatei*(datName: TDateiname; 
                              M: TMatrix; anzZeilen: LONGINT;
                              VAR ok: BOOLEAN);
(** Schreibt die Matrix <M> in eine Textdatei, deren Name in <datName> 
    angegeben wurde. Jede Zeile der Matrix muss einem String entsprechen,
    insbesondere mit 0X abgeschlossen sein.*)
(** In der Variablen <anzZeilen> wird der Prozedur mitgeteilt, wieviele Zeilen
    zu schreiben sind.*)
(** In der Variablen <ok> wird zurckgegeben, ob die Datei erfolgreich 
    geschrieben werden konnte.*)
VAR message: ARRAY 255 OF CHAR;
    resCode: INTEGER;
    zaehler: LONGINT;
    fehler:BOOLEAN;
    dat: F.Handle;
    antwort,code: CHAR;
BEGIN
    fehler:=FALSE;
    ok:= TRUE;
    IF F.Exist(datName) 
    THEN 
      D.WriteLn;
      D.WriteStr("Die Datei ");
      D.WriteStr(datName);
      D.WriteStr(" existiert schon. berschreiben (j/n)?");
      REPEAT
        D.ReadChar(antwort);
        IF (antwort#"j") & (antwort#"J") & (antwort#"n") & (antwort#"N")
        THEN D.GotoXY(D.WhereX()-1,D.WhereY());
             D.WriteChar(" ");
             D.GotoXY(D.WhereX()-1,D.WhereY());
        END;
      UNTIL (antwort="j") OR (antwort="J") OR (antwort="n") OR (antwort="N");
      IF (antwort="j") OR (antwort="J") 
      THEN
        F.Delete(datName,resCode);
        IF resCode # F.NOERROR THEN
          D.WriteLn;
          D.WriteStr("Fehler beim berschreiben der Datei: ");
          F.GetErrorMessage(resCode,message);
          D.WriteStr(message);
          fehler:=TRUE;
          ok := FALSE;
        END;
      ELSE fehler:=TRUE;
           ok := FALSE;
      END;
    END;

    IF ~fehler
    THEN 
      F.Open(datName,TRUE,F.DENYALL,F.WRITEONLY,dat,resCode);
      IF resCode # F.NOERROR
      THEN 
        D.WriteStr("Fehler beim Schreiben der Datei: ");
        F.GetErrorMessage(resCode,message);
        D.WriteStr(message);
        D.WriteLn;
        ok := FALSE;
      ELSE
        fehler:=FALSE;
        zaehler := 0;
        REPEAT
          F.WriteLn(dat,M[zaehler],resCode);
          IF resCode # F.NOERROR
          THEN 
            fehler:=TRUE;
            ok := FALSE;
            F.GetErrorMessage(resCode,message);
            D.WriteStr("Fehler beim Schreiben der Datei: ");
            D.WriteStr(message);
            D.WriteLn;
          END;
          INC(zaehler)
        UNTIL (zaehler=anzZeilen) OR fehler;
      END;
      F.Close(dat);
    END;
END SchreibeTextdatei;

END Textdatei2Matrix.
