(****************************************************************************)   
(** <MKSoftware> - Software fr den Unterricht von Michael Khn *)   
(** www.mksoftware.de.vu *)
(** mail: mksoftware@bm-kuehn.de *)
(****************************************************************************)   
(* Textdatei2String.mod (Mrz 2002) *)
(* (c) by Michael Khn *)
(**                                                                          *)
(** Dieses Service-Modul darf fr Unterrichtszwecke frei verwendet werden.   *)
(**                                                                          *)
(** Fr Verbesserungsvorschlge bin ich dankbar.                             *)
(****************************************************************************)   

(** *)
(** <Textdatei2String>*)
(** Lesen und Schreiben von Textdateien*)
(** *)
(** Dieses Modul ermglicht auf vereinfachte Weise gegenber dem File-Modul
    den Umgang mit Textdateien. Es knnen Textdateien gelesen und 
    geschrieben werden. Im Anwendungsprogramm wird der gesamte Text in einem
    String gespeichert.
*)
MODULE Textdatei2String; 
IMPORT D:=Display, F:=File;

CONST (** Die Konstante <maxlaenge> muss evtl. angepasst werden! *)
      maxlaenge- = 2000;
TYPE TDateiname* = ARRAY 255 OF CHAR;
     (** evtl. muss die Konstante <maxlaenge> in der Datei
     Textfile2String.mod angepasst werden!*)
     TString* = ARRAY maxlaenge OF CHAR;

PROCEDURE Laenge*(string: TString): LONGINT;
(** Ermittelt die Lnge eines <string>s. 
    Dabei wird das Abschlusszeichen 0X nicht mitgezhlt.*)
(** "Hallo" hat also eine Lnge von 5.*)
(** Diese Prozedur ist mitunter ntzlich im Zusammenhang mit
    diesem Dateihandling.*)
VAR i: LONGINT;
BEGIN
  i:=0;
  WHILE (string[i] # 0X) & (i<MAX(LONGINT)) DO
    INC(i);
  END;
  RETURN i;
END Laenge;

PROCEDURE LiesTextdatei*(datName: TDateiname; VAR st: TString; VAR ok: BOOLEAN);
(** Liest eine Textdatei, deren Name in <datName> angegeben wurde. 
Nach diesem Prozeduraufruf steht der Text der Datei in dem String <st>. 
Das erste Zeichen hat die Nummer 0. Die Lnge des Strings ergibt sich 
aus dem String-Endezeichen 0X.*)
(** In der Variablen <ok> wird zurckgegeben, ob die Datei erfolgreich 
    gelesen werden konnte.*)
VAR zaehler,laenge: LONGINT;
    resCode: INTEGER;
    message: ARRAY 255 OF CHAR;
    dat: F.Handle;
BEGIN
  ok := TRUE;
  zaehler:=-1;
  IF ~F.Exist(datName) 
    THEN
      D.WriteStr("Fehler! Die Datei ");
      D.WriteStr(datName);
      D.WriteStr(" existiert nicht!");
      D.WriteLn;
      ok := FALSE;
    ELSE
      F.Open(datName,FALSE,F.DENYWRITE,F.READONLY,dat,resCode);
      IF resCode # F.NOERROR 
      THEN F.GetErrorMessage(resCode,message);
           D.WriteStr("Fehler beim ffnen der Datei ");
           D.WriteStr(datName);
           D.WriteStr(": ");
           D.WriteStr(message);
           D.WriteLn;
           ok := FALSE;
      ELSE WHILE (~F.AtEnd(dat)) & (zaehler<maxlaenge-1) DO
             INC(zaehler);
             F.ReadChar(dat,st[zaehler],resCode);
             IF resCode # F.NOERROR 
             THEN F.GetErrorMessage(resCode,message);
                  D.WriteStr("Fehler beim ffnen der Datei ");
                  D.WriteStr(datName);
                  D.WriteStr(": ");
                  D.WriteStr(message);
                  D.WriteLn;
                  ok := FALSE;
             END;
           END;
           IF (zaehler=maxlaenge-1) & (~F.AtEnd(dat)) THEN
             D.WriteStr("Datei ");
             D.WriteStr(datName);
             D.WriteStr(" ist zu lang.");
             D.WriteLn;
             D.WriteStr("Es werden nur die ersten ");
             D.WriteInt(maxlaenge,1);
             D.WriteStr(" Zeichen bercksichtigt!");
             D.WriteLn;
             ok := FALSE;
           END;
      END;
      F.Close(dat);
  END;
  laenge:=zaehler+1;
  st[laenge]:=0X;
END LiesTextdatei;

PROCEDURE SchreibeTextdatei*(datName: TDateiname; st: TString; VAR ok: BOOLEAN);
(** Schreibt eine Textdatei, deren Name in <datName> angegeben wurde. Der 
Text, der geschrieben wird, steht in dem String <st>. Die Lnge dieses 
Strings wird ber das String-Endezeichen 0X erkannt.*)
(** Sollte die Datei schon existieren, wird gefragt, ob sie berschrieben
werden soll. *)
(** In der Variablen <ok> wird zurckgegeben, ob die Datei erfolgreich
    geschrieben werden konnte.*)
VAR message: ARRAY 255 OF CHAR;
    resCode: INTEGER;
    length,zaehler: LONGINT;
    fehler:BOOLEAN;
    dat: F.Handle;
    antwort,code: CHAR;
BEGIN
    ok := TRUE;
    length := Laenge(st);
    fehler:=FALSE;
    IF F.Exist(datName) 
    THEN 
      D.WriteLn;
      D.WriteStr("Die Datei ");
      D.WriteStr(datName);
      D.WriteStr(" existiert schon. berschreiben (j/n)? ");
      REPEAT
        D.ReadChar(antwort);
        IF (antwort#"j") & (antwort#"J") & (antwort#"n") & (antwort#"N")
        THEN D.GotoXY(D.WhereX()-1,D.WhereY());
             D.WriteChar(" ");
             D.GotoXY(D.WhereX()-1,D.WhereY());
        END;
      UNTIL (antwort="j") OR (antwort="J") OR (antwort="n") OR (antwort="N");
      D.WriteLn;
      IF (antwort="j") OR (antwort="J") 
      THEN
        F.Delete(datName,resCode);
        IF resCode # F.NOERROR THEN
          D.WriteLn;
          D.WriteStr("Fehler beim berschreiben der Datei: ");
          F.GetErrorMessage(resCode,message);
          D.WriteStr(message);
          D.WriteLn;
          ok := FALSE;
          fehler:=TRUE;
        END;
      ELSE fehler:=TRUE;
           ok := FALSE;
      END;
    END;

    IF ~fehler
    THEN 
      F.Open(datName,TRUE,F.DENYALL,F.WRITEONLY,dat,resCode);
      IF resCode # F.NOERROR
      THEN 
        D.WriteStr("Fehler beim Schreiben der Datei: ");
        F.GetErrorMessage(resCode,message);
        D.WriteStr(message);
        D.WriteLn;
        ok := FALSE;
      ELSE
        fehler:=FALSE;
        FOR zaehler:=0 TO length-1 DO
          F.WriteChar(dat,st[zaehler],resCode);
          IF resCode # F.NOERROR
          THEN 
            ok := FALSE;
            fehler:=TRUE; 
            F.GetErrorMessage(resCode,message);
          END;
          IF fehler
          THEN 
            D.WriteStr("Fehler beim Schreiben der Datei: ");
            D.WriteStr(message);
            D.WriteLn;
            ok := FALSE;
          END;
        END;
      END;
      F.Close(dat);
    END;
END SchreibeTextdatei;

END Textdatei2String.
